package task3;

import java.util.Iterator;
import java.util.ListIterator;
import java.util.Stack;

/**
 * Represents a pile of books.
 */
public class BookPile implements Iterable<Book> {
  private Stack<Book> books;

  /**
   * Construct an empty pile.
   */
  public BookPile() {
    books = new Stack<>();
  }

  /**
   * Add a book to the top of this Pile.
   * @param book a Book to be added to this pile.
   *             Pre: book != null
   */
  public void add(Book book) {
    books.push(book);
  }

  /**
   * Removes the book at the top of this pile and returns it.
   *          Pre: this.size() > 0
   * @return the book at the top of this pile.
   */
  public Book removeTop() {
    return books.pop();
  }

  /**
   * Get the number of books in this pile.
   * @return the number of books in this pile.
   *         Inv: size() >= 0
   */
  public int size() {
    return books.size();
  }

  @Override
  public Iterator<Book> iterator() {
    return new BookPileIterator();
  }

  @Override
  public String toString() {
    return books.toString();
  }

  private class BookPileIterator implements Iterator<Book> {
    private final ListIterator<Book> it = books.listIterator(books.size());

    @Override
    public boolean hasNext() {
      return it.hasPrevious();
    }

    @Override
    public Book next() {
      return it.previous();
    }
  }
}
